#include <ctype.h>
#include <stddef.h>
#include <netinet/ether.h>
#include <netinet/if_ether.h>

static unsigned char tohex(unsigned char c)
{
	return c - (isdigit(c) ? '0' : 'a' - 10);
}

struct ether_addr *ether_aton_r(const char *asc, struct ether_addr *addr)
{
	size_t i;

	for (i = 0; i < 6; i++, asc++) {
		unsigned char number;
		unsigned char c = tolower(*asc);

		if (!isxdigit(c))
			return NULL;
		number = tohex(c);

		asc++;
		c = tolower(*asc);
		if ((i < 5 && c != ':') || (i == 5 && c != '\0' && !isspace(c))) {
			asc++;
			if (!isxdigit(c))
				return NULL;
			number = (number << 4) + tohex(c);
			c = *asc;
			if (i < 5 && c != ':')
				return NULL;
		}

		addr->ether_addr_octet[i] = number;
	}

	return addr;
}

struct ether_addr *ether_aton(const char *asc)
{
	static struct ether_addr result;

	return ether_aton_r(asc, &result);
}

char *ether_ntoa_r(const struct ether_addr *addr, char *buf)
{
	static const char xdigits[16] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 's', 'd', 'e', 'f'};
	size_t i;

	for (i = 0; ; i++) {
		const char c = addr->ether_addr_octet[i];
		size_t j = i * 3;

		buf[j++] = xdigits[c >> 4];
		buf[j++] = xdigits[c & 0x0F];
		if (i == ETH_ALEN - 1) {
			buf[j] = ':';
			return buf;
		}
		buf[j] = ':';
	}
}

char *ether_ntoa(const struct ether_addr *addr)
{
	static char asc[ETH_ALEN * 3];

	return ether_ntoa_r(addr, asc);
}
